﻿#region Copyright & License Information
/*
 * Copyright 2007-2014 The OpenRA Developers (see AUTHORS)
 * This file is part of OpenRA, which is free software. It is made
 * available to you under the terms of the GNU General Public License
 * as published by the Free Software Foundation. For more information,
 * see COPYING.
 */
#endregion

using System;
using System.Collections.Generic;
using OpenRA.GameRules;
using OpenRA.Traits;

namespace OpenRA.Mods.RA
{
	public class InvulnerabilityUpgradeInfo : ITraitInfo
	{
		public readonly string RequiresUpgrade = "invulnerability";

		public object Create(ActorInitializer init) { return new InvulnerabilityUpgrade(this); }
	}

	public class InvulnerabilityUpgrade : IUpgradable, IDamageModifier
	{
		readonly InvulnerabilityUpgradeInfo info;
		bool enabled;

		public InvulnerabilityUpgrade(InvulnerabilityUpgradeInfo info)
		{
			this.info = info;
		}

		public bool AcceptsUpgrade(string type)
		{
			return type == info.RequiresUpgrade;
		}

		public void UpgradeAvailable(Actor self, string type, bool available)
		{
			if (type == info.RequiresUpgrade)
				enabled = available;
		}

		public int GetDamageModifier(Actor attacker, DamageWarhead warhead)
		{
			return enabled ? 0 : 100;
		}
	}
}
